@@ -1,6 +1,11 @@
 context("checkInteger")
 
 test_that("checkInteger", {
+  myobj = 1L
+  expect_succ(Integer, myobj)
+  myobj = 1
+  expect_fail(Integer, myobj)
+
   expect_true(testInteger(integer(0)))
   expect_false(testInteger(NULL))
   expect_false(testInteger(TRUE))
@@ -13,6 +18,5 @@ test_that("checkInteger", {
   expect_true(testInteger(1:3, lower = 1L, upper = 3L))
   expect_false(testInteger(1:3, lower = 5))
 
-  expect_true(assertInteger(1L))
   expect_error(assertInteger(1), "integer")
 })
